//          File: prim.cpp
//        Author: Fred Sisenda
// Last Modified: March 19, 2012


#include <algorithm>
#include <vector>
#include <limits>
#include <iostream>
#include "mst.h"
#include "vertex.h"
#include "graph.h"

using namespace std;

// Forward decalarations
class Edge;

// Comparator for vertices
// Useful for max heap
class CompareVertices{
	public:
		bool operator()(Vertex* v1, Vertex* v2){
			if (v1->data > v2->data) return true;
			return false;
		}
};


// Prim's algorithm for minimum spanning tree
// Dasgupta pg. 152
vector<Edge*> Prim::MST(Graph& g){
	clock_t s_time = clock(), dur;
	int MAX = numeric_limits<int>::max();
	unsigned int n = g.getNumVertices();
	vector<Edge*> prev(n-1);	// start won't have prev
	vector<Vertex*> H(n);
	
	for (unsigned int ii = 0; ii<n; ii++){
		Vertex* v = g.getVertexByIndex(ii);
		H[ii] = v;
		v->data = MAX;
	}
	
	Vertex* start = g.getVertexByIndex(0);
	start->data = 0;
	
	make_heap (H.begin(), H.end(), CompareVertices());
	while (!H.empty()){
		Vertex* top = H.front();
		pop_heap (H.begin(), H.end(), CompareVertices());
		H.pop_back();
		int need_heapify = 0;
		for (int ii = 0; ii<top->getNumEdges(); ii++){
			int edgeWeight = top->getEdgeWeight(ii);
			Vertex* dest = top->getEdgeDest(ii);
			if (dest->data > edgeWeight){
				dest->data = edgeWeight;
				int destIndex = dest->getIndex();
				prev[destIndex-1] = top->getEdgeObj(ii);
				need_heapify = 1;
			}
		}
		if (need_heapify==1) make_heap (H.begin(), H.end(), CompareVertices());
	}
	
	dur = clock() - s_time;
	cout << "Prim's algorithm: " << endl;
	cout << "\tTime: " << dur << endl;
	
	return prev;
}


